package com.zh.lingnuo.controller;


import com.zh.lingnuo.service.IUserService;
import com.zh.lingnuo.service.IUserTokenService;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-08
 */
@Tag(name = "登录")
@RestController
public class UserTokenController {

    @Autowired
    private IUserService userService;
    @Autowired
    private IUserTokenService userTokenService;

    @Data
    public static class ParamsLoginUser{
        @Schema(title = "账号")
        @NotNull(message = "账号不能为空")
        private String name;
        @Schema(title = "密码")
        @NotNull(message = "密码不能为空")
        private String password;
        @Schema(title = "类型")
        @NotNull(message = "类型不能为空")
        @Min(value = 1,message = "类型范围为1 2 3")
        @Max(value = 3,message = "类型范围为1 2 3")
        private Integer kind;
    }

    @Data
    @Accessors(chain = true)
    public static class ResultsLoginUser{
        @Schema(title = "姓名")
        private String name;
        @Schema(title = "用户类型")
        private Integer kind;
        @Schema(title = "用户类型名称")
        private String kindName;
        @Schema(title = "登录令牌")
        private String token;
    }

    @Operation(summary = "登录")
    @PostMapping("/login")
    public Result<ResultsLoginUser> login(@Validated @RequestBody ParamsLoginUser params) {
        return  userTokenService.login(params);
    }

    @Operation(summary = "退出登录")
    @PostMapping("/logout")
    public Result logout(@RequestHeader String token) {
        return  userTokenService.logout(token);
    }

}
