package com.zh.lingnuo.controller.answerer;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.controller.company.CompanyQnTitleController;
import com.zh.lingnuo.mapper.CompanyQnAnswererMapper;
import com.zh.lingnuo.model.*;
import com.zh.lingnuo.service.ICompanyQnAnswerService;
import com.zh.lingnuo.service.ICompanyQnAnswererService;
import com.zh.lingnuo.service.ICompanyQnTitleService;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */

@Tag(name = "答题管理")
@RestController
@RequestMapping("/answer")
public class CompanyQnAnswerController {

    @Autowired
    private ICompanyQnAnswerService companyQnAnswerService;

    @Autowired
    private ICompanyQnAnswererService companyQnAnswererService;

    @Autowired
    private ICompanyQnTitleService companyQnTitleService;

    @Autowired
    private CompanyQnAnswererMapper companyQnAnswererMapper;

    @Data
    public static class paramsAnswererQnList {
        private int pageNum;
        private int pageSize;
        @JsonIgnore(value = false)
        private Answerer userInfo;
    }

    @Operation(summary = "问卷列表")
    @PostMapping("/qnList")
    @Token(requireUser = 1, level = {3})
    public Result<List<CompanyQnTitleController.ResultQnTitleList>> qnList(@Validated @RequestBody paramsAnswererQnList params) {
        System.out.println(params.toString());
        return companyQnAnswerService.qnList(params,params.getUserInfo().getId());
    }

    @Data
    public static class ParamsAnswererQnOpr {
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @JsonIgnore(value = false)
        private Answerer userInfo;
    }

    @Operation(summary = "题目列表")
    @PostMapping("/list")
    @Token(requireUser = 1, level = {3})
    public Result<List<CompanyQnTitleController.ResultQnTitleList>> list(@Validated @RequestBody ParamsAnswererQnOpr params) {
        Integer qnId = params.getQnId();
        String qnErr = companyQnAnswererService.validAnswererQn(params.getUserInfo().getId(), qnId);
        if (null != qnErr) {
            return Result.err(qnErr);
        }
        return companyQnTitleService.qnTitleList(qnId);
    }

    @Data
    public static class ParamsAnswererOpr {
        @JsonIgnore(value = false)
        private Answerer userInfo;
    }

    @Data
    public static class ResultAnswerAll {
        private Integer id;
        private Integer qnId;
        private Integer titleId;
        private String title;
        private Integer titleKind;
        private String titleKindName;
        private String answerOpt;
        private String answerCont;
        private HashMap<String, List<Integer>> answerPartnerChoice;
        private String[] answerMultiOpt;
    }

    @Operation(summary = "答题结果")
    @PostMapping("/all")
    @Token(requireUser = 1, level = {3})
    public Result all(@Validated @RequestBody ParamsAnswererQnOpr params) {
        return companyQnAnswerService.all(params.getQnId(),params.getUserInfo().getId());
    }

    @Data
    public static class ResultQnTowards{
            private Integer id;
            private String name;
            private String department;
    }

    @Operation(summary = "全部被评人")
    @PostMapping("/allQnTowards")
    @Token(requireUser = 1, level = {3})
    public Result allQnTowards(@Validated @RequestBody ParamsAnswererQnOpr params) {
        return companyQnAnswerService.allQnTowards(params.getQnId(), params.getUserInfo().getId());
    }

    @Data
    public static class paramsCompanyQnAnswerAdd {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer titleId;
        @Schema(title = "填空题回答结果")
        private String answerFill;
        @Schema(title = "选择题回答结果")
        private String answerChoice;
        @Schema(title = "互选题回答结果")
        private HashMap<String, int[]> answerPartnerChoice;
        @Schema(title = "多选题回答结果")
        private String[] answerMultiChoice;
        @JsonIgnore(value = false)
        private Answerer userInfo;
    }

    @Operation(summary = "答题")
    @PostMapping("/add")
    @Token(requireUser = 1, level = {3})
    public Result add(@Validated @RequestBody paramsCompanyQnAnswerAdd params) {
        return companyQnAnswerService.add(params);
    }

    @Operation(summary = "开始答题")
    @PostMapping("/start")
    @Token(requireUser = 1, level = {3})
    public Result start(@Validated @RequestBody ParamsAnswererQnOpr params) {
        if (!companyQnAnswererService.update(new CompanyQnAnswerer(),
            new UpdateWrapper<CompanyQnAnswerer>()
                .eq("qn_id",params.getQnId())
                .eq("answerer_id", params.getUserInfo().getId())
                .eq("status", CompanyQnAnswerer.CompanyQnAnswerStatusWaiting)
                .set("status", CompanyQnAnswerer.CompanyQnAnswerStatusProgressing)
                .set("started_at", LocalDateTime.now())
        )) {
            return Result.err("提交失败");
        }
        return Result.success("");
    }

    @Operation(summary = "结束答题")
    @PostMapping("/end")
    @Token(requireUser = 1, level = {3})
    public Result end(@Validated @RequestBody ParamsAnswererQnOpr params) {
        return companyQnAnswerService.end(params.getQnId(), params.getUserInfo().getId());
    }
}
