package com.zh.lingnuo.controller.company;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.zh.lingnuo.annotation.ExcelParam;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.mapper.CompanyQnAnswerMapper;
import com.zh.lingnuo.model.Company;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.model.CompanyQnAnswerer;
import com.zh.lingnuo.model.CompanyQnTitle;
import com.zh.lingnuo.service.ICompanyQnAnswerService;
import com.zh.lingnuo.service.ICompanyQnService;
import com.zh.lingnuo.service.ICompanyQnTitleService;
import com.zh.lingnuo.serviceImpl.MailServiceImpl;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Tag(name = "问卷管理")
@RestController
@RequestMapping("/company_qn")
public class CompanyQnController {

    @Autowired
    private ICompanyQnService companyQnService;
    @Autowired
    private ICompanyQnTitleService companyQnTitleService;
    @Autowired
    private ICompanyQnAnswerService companyQnAnswerService;

    @Data
    public static class paramsQnList {
        private int pageNum;
        private int pageSize;
        @Schema(title = "名称")
        private String title;
        @Schema(title = "状态", description = "0=未投放 1 =投放中 2=已结束")
        private Integer status;
        @Schema(title = "更新时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeStart;
        @Schema(title = "更新时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeEnd;
        @Schema(title = "创建时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeStart;
        @Schema(title = "创建时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeEnd;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Data
    public static class ResultCompanyQnList {
        private Integer id;
        private Integer companyId;
        private String companyName;
        private String title;
        private String subtitle;
        private Integer status;
        private String statusName;
        private List<CompanyQnAnswerCount> count;
        private Integer isSecret;
        private LocalDateTime releaseAt;
        private LocalDateTime endAt;
        private LocalDateTime updatedAt;
        private LocalDateTime createdAt;
    }

    @Data
    public static class CompanyQnAnswerCount {
        private Integer status;
        private String statusName;
        private Integer num;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser = 1, level = {2})
    public Result<IPage<CompanyQn>> list(@Validated @RequestBody paramsQnList params) {
        return companyQnService.qnList(params);
    }

    @Data
    public static class ResultCompanyQnDetail {
        private CompanyQn baseInfo;
        private List titleList;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    @Token(requireUser = 1, level = {2})
    public Result detail(@Validated @RequestBody ParamsQnOpr params) {
        Integer qnId = params.getQnId();
        CompanyQn companyQn = companyQnService.getById(qnId);
        if (null == companyQn) {
            return Result.err("未知问卷");
        }
        if (companyQn.getCompanyId() != params.getUserInfo().getId()) {
            return Result.err("错误问卷");
        }
        Result<List<CompanyQnTitleController.ResultQnTitleList>> listResult = companyQnTitleService.qnTitleList(qnId);
        ResultCompanyQnDetail result = new ResultCompanyQnDetail();
        result.setBaseInfo(companyQn);
        if (listResult.isSuccess()) {
            result.setTitleList(listResult.getData());
        }
        return Result.success(result);
    }

    @Data
    public static class paramsQnAdd {
        @Schema(title = "名称")
        @NotNull(message = "名称不能为空")
        @NotEmpty(message = "名称不能为空")
        private String title;
        @Schema(title = "副标题")
        private String subtitle;
        @Schema(title = "是否匿名")
        @NotNull(message = "是否匿名不能为空")
        @Min(value = 0, message = "类型范围为0 1")
        @Max(value = 1, message = "类型范围为0 1")
        private Integer isSecret;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "新增")
    @PostMapping("/add")
    @Token(requireUser = 1, level = {2})
    public Result add(@Validated @RequestBody paramsQnAdd params) {
        return companyQnService.qnAdd(params);
    }

    @Data
    public static class paramsQnUpd {
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @Schema(title = "名称")
        private String title;
        @Schema(title = "副标题")
        private String subtitle;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "修改基础信息")
    @PostMapping("/upd")
    @Token(requireUser = 1, level = {2})
    public Result upd(@Validated @RequestBody paramsQnUpd params) {
        String companyErr = companyQnService.validCompany(params.getQnId(), params.getUserInfo().getId());
        if (null != companyErr) {
            return Result.err(companyErr);
        }
        return companyQnService.qnUpd(params);
    }

    @Data
    public static class paramCompanyQnRemoveAll {
        @Schema(title = "qnId")
        @NotNull(message = "qnId不能为空")
        private Integer qnId;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "清空题目")
    @PostMapping("/removeAll")
    @Token(requireUser = 1, level = {2})
    public Result removeAll(@Validated @RequestBody paramCompanyQnRemoveAll params) {
        CompanyQn companyQn = companyQnService.getById(params.getQnId());
        if (null == companyQn) {
            return Result.err("未知问卷");
        }
        if (companyQn.getCompanyId() != params.getUserInfo().getId()) {
            return Result.err("错误问卷");
        }
        if (companyQn.getStatus() != CompanyQn.CompanyQnStatusNotRelease) {
            return Result.err("已发布问卷不能修改");
        }
        if (!companyQnTitleService.remove(new QueryWrapper<CompanyQnTitle>().eq("qn_id", params.getQnId()))) {
            return Result.err("删除失败");
        }
        return Result.success("");
    }

    @Data
    public static class ParamsQnOpr {
        @Schema(title = "qnId")
        @NotNull(message = "qnId不能为空")
        private Integer qnId;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "发布")
    @PostMapping("/release")
    @Token(requireUser = 1, level = {2})
    public Result release(@Validated @RequestBody ParamsQnOpr params) {
        int qnId = params.getQnId();
        CompanyQn qn = companyQnService.getById(qnId);
        if (null == qn || qn.getCompanyId() != params.getUserInfo().getId()) {
            return Result.err("无效操作");
        }
        if (CompanyQn.CompanyQnStatusNotRelease != qn.getStatus()) {
            return Result.err("不在待发布状态");
        }
        return companyQnService.release(qn);
    }

    @Operation(summary = "全量发送邮件")
    @PostMapping("/groupSendMail")
    @Token(requireUser = 1, level = {2})
    public Result groupSendMail(@Validated @RequestBody ParamsQnOpr params) {
        return companyQnService.groupSendMail(params.getQnId(),params.getUserInfo());
    }

    @Operation(summary = "停止问卷")
    @PostMapping("/end")
    @Token(requireUser = 1, level = {2})
    public Result end(@Validated @RequestBody ParamsQnOpr params) {
        int qnId = params.getQnId();
        CompanyQn qn = companyQnService.getById(qnId);
        if (null == qn || qn.getCompanyId() != params.getUserInfo().getId() || CompanyQn.CompanyQnStatusRelease != qn.getStatus()) {
            return Result.err("无效操作");
        }
        return companyQnService.end(qnId);
    }

    @Data
    public static class ResultCompanyQn {
        private CountResult count;
        private List<ResultCompanyQnAnswer> list;
    }

    @Data
    @Accessors(chain = true)
    public static class CountResult {
        private int completedNum;
        private int progressingNum;
        private int waitingNum;
    }

    @Data
    public static class ResultCompanyQnAnswer {
        private int id;
        private String name;
        private int status;
        private String statusName;
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime startedAt;
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime finishedAt;
    }

    @Operation(summary = "问卷结果")
    @PostMapping("/result")
    @Token(requireUser = 1, level = {2})
    public Result<ResultCompanyQn> list(@Validated @RequestBody ParamsQnOpr params) {
        String companyErr = companyQnService.validCompany(params.getQnId(), params.getUserInfo().getId());
        if (null != companyErr) {
            return Result.err(companyErr);
        }
        return companyQnService.result(params.getQnId());
    }

    @Data
    public static class paramsPartnerChoiceResult {
        @Schema(title = "问卷id")
        @NotNull(message = "qnId不能为空")
        private Integer qnId;
        private int pageNum;
        private int pageSize;
    }

    @Data
    public static class AnswererResult {
        private Integer answererId;
        private String answererName;
        private Integer qnTowardsId;
        private String qnTowardsName;
        private List<AnswererTitleResult> answerTitleResult;
    }

    @Data
    @Accessors(chain = true)
    public static class AnswererTitleResult {
        private Integer titleId;
        private String answerOpt;
    }

    @Operation(summary = "答题人互评详情")
    @PostMapping("/partnerChoiceResult")
    @Token(requireUser = 1, level = {2})
    public Result partnerChoiceResult(@Validated @RequestBody paramsPartnerChoiceResult params) {
        return companyQnAnswerService.partnerChoiceResult(PageUtil.SetPage(params.getPageNum(),params.getPageSize()),params.getQnId(),false);
    }

    @Operation(summary = "答题人互评结果导出Excel")
    @PostMapping("/resultExportExcel")
    @Token(requireUser = 1, level = {2})
    public void resultExportExcel(HttpServletResponse response, @Validated @RequestBody paramsPartnerChoiceResult params) {
        companyQnAnswerService.resultExportExcel(response, params.getQnId(),false);
    }

}
