package com.zh.lingnuo.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.model.QnTitle;
import com.zh.lingnuo.service.IQnTitleService;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Tag(name = "问卷")
@RestController
@RequestMapping("/qn_title")
public class QnTitleController {

    @Autowired
    private IQnTitleService qnTitleService;

    @Data
    public static class paramsTitleList{
        private int pageNum;
        private int pageSize;
        @Schema(title = "名称")
        private String title;
        @Schema(title = "分类名称")
        private String groupTitle;
        @Schema(title = "分类id")
        private Integer groupId;
        @Schema(title = "更新时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeStart;
        @Schema(title = "更新时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeEnd;
        @Schema(title = "创建时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeStart;
        @Schema(title = "创建时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeEnd;
    }

    @Data
    public static class resultTitleList{
        private int id;
        @Schema(title = "名称")
        private String title;
        @Schema(title = "类型")
        private Integer kind;
        @Schema(title = "分类名称")
        private String kindName;
        @Schema(title = "分类id")
        private Integer groupId;
        @Schema(title = "分类名称")
        private String groupName;
        @Schema(title = "标签")
        private String tag;
        @Schema(title = "状态")
        private int status;
        @Schema(title = "选项")
        private JSONObject options;
        @Schema(title = "状态名称")
        private String statusName;
        @Schema(title = "更新时间")
        private LocalDateTime updatedAt;
        @Schema(title = "创建时间")
        private LocalDateTime createdAt;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser = 1, level = {1,2})
    public Result<IPage<resultTitleList>> list(@Validated @RequestBody paramsTitleList params) {
        return qnTitleService.titleList(params);
    }

    @Data
    public static class paramsTitleAdd {
        @Schema(title = "标题")
        @NotNull(message = "标题不能为空")
        @NotEmpty(message = "标题不能为空")
        private String title;
        @Schema(title = "分类")
        @NotNull(message = "分类不能为空")
        private Integer groupId;
        @Schema(title = "标签")
        private String tag;
        @Schema(title = "题型")
        @NotNull(message = "题型不能为空")
        @Min(value = 1, message = "题型范围为1填空2选择3互选4多选")
        @Max(value = 4, message = "题型范围为1填空2选择3互选4多选")
        private Integer kind;
        @Schema(title = "选项")
        private String[] options;
    }

    @Operation(summary = "添加")
    @PostMapping("/add")
    @Token(requireUser=1,level = {1})
    public Result add(@Validated @RequestBody paramsTitleAdd params) {
        return qnTitleService.titleAdd(params);
    }

    @Data
    public static class paramsTitleUpd {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
        @Schema(title = "标题")
        private String title;
        @Schema(title = "分类")
        private Integer groupId;
        @Schema(title = "标签")
        private String tag;
        @Schema(title = "题型")
        @Min(value = 1, message = "题型范围为1填空2选择3互选4多选")
        @Max(value = 4, message = "题型范围为1填空2选择3互选4多选")
        private Integer kind;
        @Schema(title = "选项")
        private String[] options;
    }

    @Operation(summary = "修改")
    @PostMapping("/upd")
    @Token(requireUser=1,level = {1})
    public Result upd(@Validated @RequestBody paramsTitleUpd params) {
        return qnTitleService.titleUpd(params);
    }

    @Data
    public static class paramsTitleDel {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
    }

    @Operation(summary = "删除")
    @PostMapping("/del")
    @Token(requireUser=1,level = {1})
    public Result del(@Validated @RequestBody paramsTitleDel params) {
        if (!qnTitleService.removeById(params.getId())) {
            return Result.err("错误");
        }
        return Result.success("");
    }

}
