package com.zh.lingnuo.mapper;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zh.lingnuo.controller.company.CompanyQnController;
import com.zh.lingnuo.model.CompanyQn;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.zh.lingnuo.model.CompanyQnAnswer;
import lombok.Data;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
public interface CompanyQnMapper extends BaseMapper<CompanyQn> {

    @Data
    public static class MapperResultCompanyQn{
        private Integer id;
        private Integer companyId;
        private String companyName;
        private String title;
        private String subtitle;
        private Integer status;
        private Integer isSecret;
        private Integer sumAnswererStatusWaiting;
        private Integer sumAnswererStatusProgressing;
        private Integer sumAnswererStatusCompleted;
        private Integer sumAnswererTotal;
        private LocalDateTime releaseAt;
        private LocalDateTime endAt;
        private LocalDateTime updatedAt;
        private LocalDateTime createdAt;
    }

    // 企业问卷列表
    @Select("SELECT t1.*,\n" +
        "(select count(*) FROM company_qn_answerer as t WHERE t1.id = t.qn_id and t.status =0) as sumAnswererStatusWaiting,\n" +
        "(select count(*) FROM company_qn_answerer as t WHERE t1.id = t.qn_id and t.status =1) as sumAnswererStatusProgressing,\n" +
        "(select count(*) FROM company_qn_answerer as t WHERE t1.id = t.qn_id and t.status =2) as sumAnswererStatusCompleted,\n" +
        "(select count(*) FROM company_qn_answerer as t WHERE t1.id = t.qn_id ) as sumAnswererTotal,\n" +
        "t2.name as companyName \n" +
        "from company_qn t1 \n" +
        "LEFT JOIN company t2 on t1.company_id=t2.id \n" +
        "${ew.customSqlSegment} " +
        "GROUP BY id ORDER BY id DESC")
    IPage<MapperResultCompanyQn> companyQnList(IPage page, @Param("ew") Wrapper<CompanyQn> qw);

    @Data
    public static class MapperResultAnswererQnList{
        private Integer id;
        private String title;
        private String subtitle;
        private Integer status;
        private Integer isSecret;
        private Integer answererStatus;
        private Integer completedSum;
        private Integer totalSum;
        private LocalDateTime releaseAt;
        private LocalDateTime endAt;
        private LocalDateTime updatedAt;
        private LocalDateTime createdAt;
    }

    // 答题人问卷列表
    @Select("SELECT t1.*,t2.status as answererStatus,COUNT(distinct t3.title_id) as completedSum,COUNT(DISTINCT t4.id) as totalSum " +
        "FROM company_qn t1\n" +
        "LEFT JOIN company_qn_answerer t2 on t1.id = t2.qn_id " +
        "LEFT JOIN company_qn_answer t3 on t3.answerer_id = t2.answerer_id\n" +
        "LEFT JOIN company_qn_title t4 on t1.id = t4.qn_id  \n" +
        "WHERE t1.status != 0 and t2.answerer_id = #{answererId} " +
        "GROUP BY t1.id ORDER BY t1.id DESC")
    IPage<MapperResultAnswererQnList> qnList(IPage page,int answererId);
}
