package com.zh.lingnuo.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.lingnuo.controller.company.CompanyQnAnswererController;
import com.zh.lingnuo.mapper.CompanyQnAnswererMapper;
import com.zh.lingnuo.model.Answerer;
import com.zh.lingnuo.model.CompanyQnAnswerer;
import com.zh.lingnuo.controller.company.CompanyQnAnswererController.ExcelAnswerData;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.lingnuo.service.IAnswererService;
import com.zh.lingnuo.service.ICompanyQnAnswererService;
import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.ExcelRead;
import com.zh.lingnuo.utils.result.Result;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.beans.IntrospectionException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2022-09-14
 */
@Service
public class CompanyQnAnswererServiceImpl extends ServiceImpl<CompanyQnAnswererMapper, CompanyQnAnswerer> implements ICompanyQnAnswererService {

    @Autowired
    private CompanyQnAnswererMapper companyQnAnswererMapper;
    @Autowired
    private IAnswererService answererService;

    //excel导入
    @Transactional
    public Result addExcel(MultipartFile file, int qnId, int companyId) {
        List<ExcelAnswerData> result = null;
        try {
            result = (List<ExcelAnswerData>) ExcelRead.ReadExcelByAll(file, 2, -1, 1, ExcelAnswerData.class);
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (IntrospectionException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
        List<String> mailList = new ArrayList<>();
        for (ExcelAnswerData r : result) {
            Answerer v = new Answerer();
            BeanUtils.copyProperties(r, v);
            if(null == v.getMail()|| null == v.getName() || null == v.getTel() || null == v.getDepartment()){
                continue;
            }
            String salt = Common.getRandomString(6, "");
            String tel = (String) Common.ObjectFieldValue(r, "tel");
            String password = Common.md5(salt + Common.md5(tel + salt));
            v.setSalt(salt).setPassword(password).setCompanyId(companyId);
            answererService.saveOrUpdate(v,new UpdateWrapper<Answerer>().eq("mail",v.getMail()));
            mailList.add(v.getMail());
        }
        List<CompanyQnAnswerer> aList = new ArrayList();
        if(mailList.size() == 0){
            return Result.success(aList);
        }
        List<Answerer> mail = answererService.list(new QueryWrapper<Answerer>().in("mail", mailList));
        List<Integer> answererIdList = companyQnAnswererMapper.answererIdList(qnId);
        for (Answerer answerer : mail) {
            if(!answererIdList.contains(answerer.getId())) {
                aList.add(new CompanyQnAnswerer().setQnId(qnId).setAnswererId(answerer.getId()));
            }
        }
        super.saveBatch(aList);
        return Result.success(aList);
    }

    // 添加
    @Transactional
    public Result add(CompanyQnAnswererController.ParamQnAnswerAdd params, int companyId) {
        int qnId = params.getQnId();
        String salt = Common.getRandomString(6, "");
        String password = Common.md5(salt + Common.md5(params.getTel() + salt));
        Answerer answerer = new Answerer()
            .setCompanyId(companyId)
            .setName(params.getName())
            .setMail(params.getMail())
            .setTel(params.getTel())
            .setWorkAge(params.getWorkAge())
            .setDepartment(params.getDepartment())
            .setSalt(salt)
            .setPassword(password);
        UpdateWrapper<Answerer> up = new UpdateWrapper<Answerer>().eq("mail", params.getMail());
        if (!answererService.saveOrUpdate(answerer,up)) {
            return Result.err("保存错误");
        }
        if(null == answerer.getId()){
            answerer = answererService.getOne(up);
        }
        if (!super.saveOrUpdate(new CompanyQnAnswerer().setQnId(qnId).setAnswererId(answerer.getId()),
            new QueryWrapper<CompanyQnAnswerer>().eq("qn_id",qnId).eq("answerer_id",answerer.getId()))){
            return Result.err("保存错误");
        }
        return Result.success("");
    }

    // 修改
    public Result upd(CompanyQnAnswererController.ParamCompanyQnAnswerUpd params) {
        Answerer r = answererService.getById(params.getAnswererId());
        if (null == r) {
            return Result.err("未知答题人");
        }
        UpdateWrapper<Answerer> up = new UpdateWrapper<Answerer>()
            .eq("id", params.getAnswererId())
            .set(StringUtils.isNotBlank(params.getName()), "name", params.getName())
            .set(StringUtils.isNotBlank(params.getMail()), "mail", params.getMail())
            .set(StringUtils.isNotBlank(params.getTel()), "tel", params.getTel())
            .set(null != params.getWorkAge(), "work_age", params.getWorkAge())
            .set(StringUtils.isNotBlank(params.getDepartment()), "department", params.getDepartment())
            .set(StringUtils.isNotBlank(params.getPassword()), "password", params.getPassword());

        if (!answererService.update(new Answerer(), up)) {
            return Result.err("更新出错");
        }
        return Result.success("");
    }

    // 校验答题人所属问卷
    public String validAnswererQn(int answererId, int qnId) {
        CompanyQnAnswerer answerer = super.getOne(new QueryWrapper<CompanyQnAnswerer>()
            .eq("qn_id",qnId).eq("answerer_id",answererId).last("limit 1"));
        if (null == answerer) {
            return "未知答题人";
        }
        return null;
    }

}
