package com.zh.lingnuo.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.lingnuo.controller.manage.CompanyController;
import com.zh.lingnuo.controller.manage.CompanyController.resultCompany;
import com.zh.lingnuo.model.Company;
import com.zh.lingnuo.mapper.CompanyMapper;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.service.ICompanyQnService;
import com.zh.lingnuo.service.ICompanyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.result.Result;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Service
public class CompanyServiceImpl extends ServiceImpl<CompanyMapper, Company> implements ICompanyService {

    @Autowired
    private ICompanyQnService companyQnService;

    // 列表
    public Result<IPage<resultCompany>> CompanyList(CompanyController.paramsCompanyList params) {
        QueryWrapper<Company> qw = new QueryWrapper<>();
        qw.like(StringUtils.isNotBlank(params.getName()), "name", params.getName())
            .ge(null != params.getCreateTimeStart(), "created_at", params.getCreateTimeStart())
            .le(null != params.getCreateTimeEnd(), "created_at", params.getCreateTimeEnd())
            .ge(null != params.getUpdateTimeStart(), "updated_at", params.getUpdateTimeStart())
            .le(null != params.getUpdateTimeEnd(), "updated_at", params.getUpdateTimeEnd())
        .orderByDesc("id");
        IPage page = super.page(PageUtil.SetPage(params.getPageNum(), params.getPageSize()), qw);
        page.convert(r -> {
            resultCompany v = new resultCompany();
            BeanUtils.copyProperties(r, v);
            v.setStatusName(Company.getStatusName((Integer) Common.ObjectFieldValue(r, "status")));
            return v;
        });
//        page.convert(User -> ConvertUtils.beanCopy(User, UserVo.class));
        return Result.success(page);
    }

    // 添加
    public Result CompanyAdd(CompanyController.paramsCompanyAdd params) {
        QueryWrapper<Company> qw = new QueryWrapper<>();
        qw.eq("name", params.getName())
            .or().eq("mail", params.getMail())
            .or().eq("tel", params.getTel())
            .last("limit 1");
        if (null != super.getOne(qw)) {
            return Result.err("所填信息已存在，请勿重复");
        }
        String salt = Common.getRandomString(6, "");
        String password = Common.md5(salt + Common.md5(params.getPassword() + salt));
        super.save(new Company()
            .setName(params.getName())
            .setMail(params.getMail())
            .setTel(params.getTel())
            .setSalt(salt)
            .setPassword(password)
            .setStatus(1));
        return Result.success("");
    }

    // 详情
    public Result<resultCompany> CompanyInfo(CompanyController.paramsCompanyInfo params) {
        resultCompany resultCompany = new resultCompany();
        Company company = super.getById(params.getId());
        if (null == company) {
            return Result.err("无结果");
        }
        BeanUtils.copyProperties(company, resultCompany);
        resultCompany.setStatusName(Company.getStatusName(company.getStatus()));
        List<CompanyQn> qnList = companyQnService.list(new QueryWrapper<CompanyQn>().eq("company_id", params.getId()));
        resultCompany.setQnList(qnList);
        return Result.success(resultCompany);
    }

    // 启用停用
    public Result CompanyAble(CompanyController.paramsCompanyAble params) {
        if (!super.updateById(new Company().setStatus(params.getStatus()).setId(params.getId()))) {
            return Result.err("无效操作");
        }
        return Result.success("");
    }

}
